# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['creole',
 'creole.emitter',
 'creole.html_tools',
 'creole.parser',
 'creole.rest_tools',
 'creole.shared',
 'creole.tests',
 'creole.tests.utils']

package_data = \
{'': ['*']}

install_requires = \
['docutils']

entry_points = \
{'console_scripts': ['creole2html = creole.cmdline:cli_creole2html',
                     'html2creole = creole.cmdline:cli_html2creole',
                     'html2rest = creole.cmdline:cli_html2rest',
                     'html2textile = creole.cmdline:cli_html2textile',
                     'publish = creole.publish:publish',
                     'update_rst_readme = '
                     'creole.setup_utils:update_creole_rst_readme']}

setup_kwargs = {
    'name': 'python-creole-atlassian',
    'version': '1.4.92',
    'description': 'python-creole is an open-source (GPL) markup converter in pure Python for: creole2html, html2creole, html2ReSt, html2textile',
    'long_description': '===================\nabout python-creole\n===================\n\npython-creole is a OpenSource (GPL) Python lib for converting markups.\npython-creole is pure python. No external libs needed.\n\nCompatible Python Versions (see `tox.ini <https://github.com/jedie/python-creole/blob/master/tox.ini>`_ or `.travis.yml <https://github.com/jedie/python-creole/blob/master/.travis.yml>`_):\n\n* 3.9, 3.8, 3.7, 3.6\n\n* PyPy3\n\nExisting converters:\n\n* creole -> html\n\n* html -> creole markup\n\n* reSt -> html (for clean html code)\n\n* html -> reStructuredText markup (only a subset of reSt supported)\n\n* html -> textile markup (not completed yet)\n\nThe creole2html part based on the creole markup parser and emitter from the MoinMoin project by Radomir Dopieralski and Thomas Waldmann.\n\n+-----------------------------------+\n| |Build Status on github|          |\n+-----------------------------------+\n| |Build Status on travis-ci.org|   |\n+-----------------------------------+\n| |Coverage Status on coveralls.io| |\n+-----------------------------------+\n| |Status on landscape.io|          |\n+-----------------------------------+\n| |PyPi version|                    |\n+-----------------------------------+\n\n.. |Build Status on github| image:: https://github.com/jedie/python-creole/workflows/test/badge.svg?branch=master\n.. |Build Status on travis-ci.org| image:: https://travis-ci.org/jedie/python-creole.svg\n.. _travis-ci.org/jedie/python-creole: https://travis-ci.org/jedie/python-creole/\n.. |Coverage Status on coveralls.io| image:: https://coveralls.io/repos/jedie/python-creole/badge.svg\n.. _coveralls.io/r/jedie/python-creole: https://coveralls.io/r/jedie/python-creole\n.. |Status on landscape.io| image:: https://landscape.io/github/jedie/python-creole/master/landscape.svg\n.. _landscape.io/github/jedie/python-creole/master: https://landscape.io/github/jedie/python-creole/master\n.. |PyPi version| image:: https://badge.fury.io/py/python-creole.svg\n.. _pypi.org/project/python-creole/: https://pypi.org/project/python-creole/\n\n=======\ninstall\n=======\n\nPython packages available on: `http://pypi.python.org/pypi/python-creole/ <http://pypi.python.org/pypi/python-creole/>`_\n\n::\n\n    ~$ pip install python-creole\n\nTo setup a virtualenv via Poetry, see ``unittests`` section below.\n\n------------\ndependencies\n------------\n\nFor the most parts (``creole2html`` and ``html2creole``) no external libraries are needed.\n\nFor all functionality (and running the unittests) these modules are needed:\n\n* `docutils <http://pypi.python.org/pypi/docutils/>`_ (for the ReStructuredText stuff)\n\n* `textile <http://pypi.python.org/pypi/textile/>`_ (for html2textile tests)\n\n=======\nexample\n=======\n\n-----------\ncreole2html\n-----------\n\nConvert creole markup to html code:\n\n::\n\n    >>> from creole import creole2html\n    >>> creole2html("This is **creole //markup//**")\n    u\'<p>This is <strong>creole <i>markup</i></strong></p>\\n\'\n\n-----------\nhtml2creole\n-----------\n\nConvert html code back into creole markup:\n\n::\n\n    >>> from creole import html2creole\n    >>> html2creole(u\'<p>This is <strong>creole <i>markup</i></strong></p>\\n\')\n    u\'This is **creole //markup//**\'\n\n---------\nrest2html\n---------\n\nConvert ReStructuredText into clean html code (needs `docutils`_):\n\n::\n\n    >>> from creole.rest2html.clean_writer import rest2html\n    >>> rest2html(u"A ReSt link to `PyLucid CMS <http://www.pylucid.org>`_ :)")\n    u\'<p>A ReSt link to <a href="http://www.pylucid.org">PyLucid CMS</a> :)</p>\\\\n\'\n\n(more information: `rest2html wiki page <https://github.com/jedie/python-creole/wiki/rest2html>`_)\n\n---------\nhtml2rest\n---------\n\nConvert html code into ReStructuredText markup:\n\n::\n\n    >>> from creole import html2rest\n    >>> html2rest(u\'<p>This is <strong>ReStructuredText</strong> <em>markup</em>!</p>\')\n    u\'This is **ReStructuredText** *markup*!\'\n\n------------\nhtml2textile\n------------\n\nConvert html code into textile markup\n\n::\n\n    >>> from creole import html2textile\n    >>> html2textile(u\'<p>This is <strong>textile <i>markup</i></strong>!</p>\')\n    u\'This is *textile __markup__*!\'\n\nSee also: `http://github.com/jedie/python-creole/blob/master/demo.py <http://github.com/jedie/python-creole/blob/master/demo.py>`_\n\n=====================\nImage size additional\n=====================\n\nYou can pass image width/height in image tags, e.g.:\n\n::\n\n    >>> from creole import creole2html\n    >>> creole_markup="""{{foobar.jpg|image title|90x160}}"""\n    >>> creole2html(creole_markup)\n    \'<p><img src="foobar.jpg" title="image title" alt="image title" width="90" height="160" /></p>\'\n\nThe third part (``90x160``) is not in creole standard, you can force a *strict* mode, e.g.:\n\n::\n\n    >>> creole2html(creole_markup, strict=True)\n    \'<p><img src="foobar.jpg" title="image title|90x160" alt="image title|90x160" /></p>\'\n\n================================\nSource code highlighting support\n================================\n\nYou can find a example macro which highlight source code thanks to the pygments\nlibrary. It is located here: `/creole/shared/example_macros.py <https://github.com/jedie/python-creole/blob/master/creole/shared/example_macros.py>`_.\nHere is how to use it:\n\n::\n\n    >>> from creole import creole2html\n    >>> from creole.shared.example_macros import code\n    >>> creole_markup="""<<code ext=".py">>#some code\\nprint(\'coucou\')\\n<</code>>"""\n    >>> creole2html(creole_markup, macros={\'code\': code})\n\n=====================\ncommandline interface\n=====================\n\nIf you have python-creole installed, you will get these simple CLI scripts:\n\n* creole2html\n\n* html2creole\n\n* html2rest\n\n* html2textile\n\nHere the ``--help`` output from ``html2creole``:\n\n::\n\n    $ html2creole --help\n    usage: html2creole [-h] [-v] [--encoding ENCODING] sourcefile destination\n    \n    python-creole is an open-source (GPL) markup converter in pure Python for:\n    creole2html, html2creole, html2ReSt, html2textile\n    \n    positional arguments:\n      sourcefile           source file to convert\n      destination          Output filename\n    \n    optional arguments:\n      -h, --help           show this help message and exit\n      -v, --version        show program\'s version number and exit\n      --encoding ENCODING  Codec for read/write file (default encoding: utf-8)\n\nExample to convert a html file into a creole file:\n\n::\n\n    $ html2creole foobar.html foobar.creole\n\n=============\ndocumentation\n=============\n\nWe store documentation/examples into the project wiki:\n\n* `https://github.com/jedie/python-creole/wiki <https://github.com/jedie/python-creole/wiki>`_\n\nHow to handle unknown html tags in html2creole:\n\n* `https://github.com/jedie/python-creole/wiki/Unknown-Html-Tags <https://github.com/jedie/python-creole/wiki/Unknown-Html-Tags>`_\n\nContributers should take a look at this page:\n\n* `https://github.com/jedie/python-creole/wiki/Developer-Info <https://github.com/jedie/python-creole/wiki/Developer-Info>`_\n\nCreole Markup Cheat Sheet can be found here: `http://www.wikicreole.org/wiki/CheatSheet <http://www.wikicreole.org/wiki/CheatSheet>`_\n\n|Creole Markup Cheat Sheet|\n\n.. |Creole Markup Cheat Sheet| image:: http://www.wikicreole.org/imageServlet?page=CheatSheet%2Fcreole_cheat_sheet.png&width=340\n\n---------\nunittests\n---------\n\n::\n\n    # clone repository (or use your fork):\n    ~$ git clone https://github.com/jedie/python-creole.git\n    ~$ cd python-creole\n    \n    # install or update poetry:\n    ~/python-creole$ make install-poetry\n    \n    # install python-creole via poetry:\n    ~/python-creole$ make install\n    \n    # Run pytest:\n    ~/python-creole$ make pytest\n    \n    # Run pytest via tox with all environments:\n    ~/python-creole$ make tox\n    \n    # Run pytest via tox with one Python version:\n    ~/python-creole$ make tox-py38\n    ~/python-creole$ make tox-py37\n    ~/python-creole$ make tox-py36\n\n------------\nmake targets\n------------\n\nTo see all make targets, just call ``make``:\n\n::\n\n    ~/python-creole$ make\n    help                 List all commands\n    install-poetry       install or update poetry\n    install              install python-creole via poetry\n    lint                 Run code formatters and linter\n    fix-code-style       Fix code formatting\n    tox-listenvs         List all tox test environments\n    tox                  Run pytest via tox with all environments\n    tox-py36             Run pytest via tox with *python v3.6*\n    tox-py37             Run pytest via tox with *python v3.7*\n    tox-py38             Run pytest via tox with *python v3.8*\n    tox-py39             Run pytest via tox with *python v3.9*\n    pytest               Run pytest\n    update-rst-readme    update README.rst from README.creole\n    publish              Release new version to PyPi\n\n--------------------\nUse creole in README\n--------------------\n\nWith python-creole you can convert a README on-the-fly from creole into ReStructuredText in setup.py\nHow to do this, read: `https://github.com/jedie/python-creole/wiki/Use-In-Setup <https://github.com/jedie/python-creole/wiki/Use-In-Setup>`_\n\nNote: In this case you must install **docutils**! See above.\n\n=======\nhistory\n=======\n\n* *dev* - `compare v1.4.9...master <https://github.com/jedie/python-creole/compare/v1.4.9...master>`_ \n\n    * TBC\n\n* v1.4.9 - 2020-11-4 - `compare v1.4.8...v1.4.9 <https://github.com/jedie/python-creole/compare/v1.4.8...v1.4.9>`_ \n\n    * Add missing classifier for Python 3.9 (`Contributed by jugmac00 <https://github.com/jedie/python-creole/pull/55>`_)\n\n    * Update readme test\n\n* v1.4.8 - 2020-10-17 - `compare v1.4.7...v1.4.8 <https://github.com/jedie/python-creole/compare/v1.4.7...v1.4.8>`_ \n\n    * Validate generated ``README.rst`` with `readme-renderer <https://pypi.org/project/readme-renderer/>`_\n\n* v1.4.7 - 2020-10-17 - `compare v1.4.6...v1.4.7 <https://github.com/jedie/python-creole/compare/v1.4.6...v1.4.7>`_ \n\n    * ``update_rst_readme()`` will touch ``README.rst`` if there are not change (timestamp will not changed in file)\n\n    * Run tests with Python 3.9, too.\n\n    * Some meta updates to project setup\n\n* v1.4.6 - 2020-02-13 - `compare v1.4.5...v1.4.6 <https://github.com/jedie/python-creole/compare/v1.4.5...v1.4.6>`_ \n\n    * less restricted dependency specification\n\n* v1.4.5 - 2020-02-13 - `compare v1.4.4...v1.4.5 <https://github.com/jedie/python-creole/compare/v1.4.4...v1.4.5>`_ \n\n    * new: ``creole.setup_utils.assert_rst_readme`` for project setup tests\n\n    * use `https://github.com/ymyzk/tox-gh-actions <https://github.com/ymyzk/tox-gh-actions>`_ on gitlab CI\n\n* v1.4.4 - 2020-02-07 - `compare v1.4.3...v1.4.4 <https://github.com/jedie/python-creole/compare/v1.4.3...v1.4.4>`_ \n\n    * Fix #44: Move ``poetry-publish`` to ``dev-dependencies`` and lower ``docutils`` requirement to |^0.15|\n\n    * some code style updated\n\n    * Always update README.rst before publish\n\n* v1.4.3 - 2020-02-01 - `compare v1.4.2...v1.4.3 <https://github.com/jedie/python-creole/compare/v1.4.2...v1.4.3>`_ \n\n    * Use new `poetry-publish <https://pypi.org/project/poetry-publish/>`_ for ``make publish``\n\n* v1.4.2 - 2020-02-01 - `compare v1.4.1...v1.4.2 <https://github.com/jedie/python-creole/compare/v1.4.1...v1.4.2>`_ \n\n    * Update CI configs on github and travis\n\n    * Update ``Makefile``: add ``make publish`` and ``make update-rst-readme``\n\n    * Add generated ``README.rst`` in repository to fix install problems about missing readme\n\n* v1.4.1 - 2020-01-19 - `compare v1.4.0...v1.4.1 <https://github.com/jedie/python-creole/compare/v1.4.0...v1.4.1>`_ \n\n    * Remove Python v2 support code\n\n    * `Fix "Undefined substitution referenced" error <https://github.com/jedie/python-creole/issues/26>`_ contributed by dforsi\n\n    * `Fix regression in tests for setup_utils <https://github.com/jedie/python-creole/pull/37>`_ contributed by jugmac00\n\n    * Fix code style with: autopep8\n\n    * sort imports with isort\n\n    * change old ``%-formatted`` and ``.format(...)`` strings into Python 3.6+\'s ``f-strings`` with flynt\n\n    * Activate linting in CI pipeline\n\n* v1.4.0 - 2020-01-19 - `compare v1.3.2...v1.4.0 <https://github.com/jedie/python-creole/compare/v1.3.2...v1.4.0>`_ \n\n    * modernize project:\n\n        * use poetry\n\n        * Add a ``Makefile``\n\n        * use pytest and tox\n\n        * remove Python v2 support\n\n        * Test with Python v3.6, v3.7 and v3.8\n\n* v1.3.2 - 2018-02-27 - `compare v1.3.1...v1.3.2 <https://github.com/jedie/python-creole/compare/v1.3.1...v1.3.2>`_ \n\n    * Adding optional img size to creole2html and html2creole contributed by `John Dupuy <https://github.com/JohnAD>`_\n\n    * run tests also with python 3.5 and 3.6\n\n* v1.3.1 - 2015-08-15 - `compare v1.3.0...v1.3.1 <https://github.com/jedie/python-creole/compare/v1.3.0...v1.3.1>`_ \n\n    * Bugfix for "Failed building wheel for python-creole"\n\n* v1.3.0 - 2015-06-02 - `compare v1.2.2...v1.3.0 <https://github.com/jedie/python-creole/compare/v1.2.2...v1.3.0>`_ \n\n    * Refactory internal file structure\n\n    * run unittests and doctests with nose\n\n    * Refactor CLI tests\n\n    * skip official support for Python 2.6\n\n    * small code cleanups and fixes.\n\n    * use **json.dumps()** instead of **repr()** in some cases\n\n* v1.2.2 - 2015-04-05 - `compare v1.2.1...v1.2.2 <https://github.com/jedie/python-creole/compare/v1.2.1...v1.2.2>`_ \n\n    * Bugfix textile unittests if url scheme is unknown\n\n    * migrate google-code Wiki to github and remove google-code links\n\n* v1.2.1 - 2014-09-14 - `compare v1.2.0...v1.2.1 <https://github.com/jedie/python-creole/compare/v1.2.0...v1.2.1>`_ \n\n    * Use origin PyPi code to check generated reStructuredText in setup.py\n\n    * Update unitest for textile v2.1.8\n\n* v1.2.0 - 2014-05-15 - `compare v1.1.1...v1.2.0 <https://github.com/jedie/python-creole/compare/v1.1.1...v1.2.0>`_ \n\n    * NEW: Add ``<<code>>`` example macro (Source code highlighting with pygments) - implemented by Julien Enselme\n\n    * NEW: Add ``<<toc>>`` macro to create a table of contents list\n\n    * Bugfix for: AttributeError: \'CreoleParser\' object has no attribute \'_escaped_char_repl\'\n\n    * Bugfix for: AttributeError: \'CreoleParser\' object has no attribute \'_escaped_url_repl\'\n\n    * API Change: Callable macros will raise a TypeError instead of create a DeprecationWarning (Was removed in v0.5)\n\n* v1.1.1 - 2013-11-08\n\n    * Bugfix: Setup script exited with error: can\'t copy \'README.creole\': doesn\'t exist or not a regular file\n\n* v1.1.0 - 2013-10-28\n\n    * NEW: Simple commandline interface added.\n\n* v1.0.7 - 2013-08-07\n\n    * Bugfix in \'clean reStructuredText html writer\' if docutils => v0.11 used.\n\n    * Bugfix for PyPy 2.1 usage\n\n* v1.0.6 - 2012-10-15\n\n    * Security fix in rest2html: Disable "file_insertion_enabled" and "raw_enabled" as default.\n\n* v1.0.5 - 2012-09-03\n\n    * made automatic protocol links more strict: Only whitespace before and at the end are allowed.\n\n    * Bugfix: Don\'t allow ``ftp:/broken`` (Only one slash) to be a link.\n\n* v1.0.4 - 2012-06-11\n\n    * html2rest: Handle double link/image substitution and raise better error messages\n\n    * Bugfix in unittests (include test README file in python package).  Thanks to Wen Heping for reporting this.\n\n* v1.0.3 - 2012-06-11\n\n    * Bugfix: ``AttributeError: \'module\' object has no attribute \'interesting_cdata\'`` from HTMLParser patch. Thanks to Wen Heping for reporting this.\n\n    * Fix a bug in get_long_description() ReSt test for Py3k and his unittests.\n\n    * Use Travis CI, too.\n\n* v1.0.2 - 2012-04-04\n\n    * Fix "`AttributeError: \'NoneType\' object has no attribute \'parent\' <https://github.com/jedie/python-creole/issues/6>`_" in html2creole.\n\n* v1.0.1 - 2011-11-16\n\n    * Fix "`TypeError: expected string or buffer <https://github.com/jedie/python-creole/issues/5>`_" in rest2html.\n\n    * `Bugfix in exception handling. <https://github.com/jedie/python-creole/commit/e8422f944709a5f8c2c6a8c8a58a84a92620f035>`_\n\n* v1.0.0 - 2011-10-20\n\n    * Change API: Replace \'parser_kwargs\' and \'emitter_kwargs\' with separate arguments. (More information on `API Wiki Page <https://github.com/jedie/python-creole/wiki/API>`_)\n\n* v0.9.2\n\n    * Turn zip_safe in setup.py on and change unittests API.\n\n* v0.9.1\n\n    * Many Bugfixes, tested with CPython 2.6, 2.7, 3.2 and PyPy v1.6\n\n* v0.9.0\n\n    * Add Python v3 support (like `http://python3porting.com/noconv.html <http://python3porting.com/noconv.html>`_ strategy)\n\n    * move unittests into creole/tests/\n\n    * Tested with Python 2.7.1, 3.2 and PyPy v1.6.1 15798ab8cf48 jit\n\n* v0.8.5\n\n    * Bugfix in html2creole: ignore links without href\n\n* v0.8.4\n\n    * Bugfix in html parser if list tag has attributes: `https://code.google.com/p/python-creole/issues/detail?id=19#c4 <https://code.google.com/p/python-creole/issues/detail?id=19#c4>`_\n\n* v0.8.3\n\n    * Better error message if given string is not unicode: `https://code.google.com/p/python-creole/issues/detail?id=19 <https://code.google.com/p/python-creole/issues/detail?id=19>`_\n\n* v0.8.2\n\n    * Bugfix in get_long_description() error handling (*local variable \'long_description_origin\' referenced before assignment*)\n\n* v0.8.1\n\n    * Bugfix for installation under python 2.5\n\n    * Note: `setup helper <https://github.com/jedie/python-creole/wiki/Use-In-Setup>`_ changed: rename ``GetLongDescription(...)`` to ``get_long_description(...)``\n\n* v0.8\n\n    * New GetLongDescription() helper for setup.py, see: `https://github.com/jedie/python-creole/wiki/Use-In-Setup`_\n\n* v0.7.3\n\n    * Bugfix in html2rest:\n\n        * table without ``<th>`` header\n\n        * new line after table\n\n        * create reference hyperlinks in table cells intead of embedded urls.\n\n        * Don\'t always use raise_unknown_node()\n\n    * Add child content to raise_unknown_node()\n\n* v0.7.2\n\n    * Activate ``----`` to ``<hr>`` in html2rest\n\n    * Update demo.py\n\n* v0.7.1\n\n    * Bugfix if docutils are not installed\n\n    * API change: rest2html is now here: ``from creole.rest2html.clean_writer import rest2html`` \n\n* v0.7.0\n\n    * **NEW**: Add a html2reStructuredText converter (only a subset of reSt supported)\n\n* v0.6.1\n\n    * Bugfix: separate lines with one space in "wiki style line breaks" mode\n\n* v0.6\n\n    * **NEW**: html2textile converter\n\n    * some **API changed**!\n\n* v0.5\n\n    * **API changed**:\n\n        * Html2CreoleEmitter optional argument \'unknown_emit\' takes now a callable for handle unknown html tags.\n\n        * No macros used as default in creole2html converting.\n\n        * We remove the support for callable macros. Only dict and modules are allowed.\n\n    * remove unknown html tags is default behaviour in html2creole converting.\n\n    * restructure and cleanup sourcecode files.\n\n* v0.4\n\n    * only emit children of empty tags like div and span (contributed by Eric O\'Connell)\n\n    * remove inter wiki links and doesn\'t check the protocol\n\n* v0.3.3\n\n    * Use <tt> when {{{ ... }}} is inline and not <pre>, see: `PyLucid Forum Thread <http://forum.pylucid.org/viewtopic.php?f=3&t=320>`_\n\n    * Bugfix in html2creole: insert newline before new list. TODO: apply to all block tags: `issues 16 <http://code.google.com/p/python-creole/issues/detail?id=16#c5>`_\n\n* v0.3.2\n\n    * Bugfix for spaces after Headline: `issues 15 <https://code.google.com/p/python-creole/issues/detail?id=15>`_\n\n* v0.3.1\n\n    * Make argument \'block_rules\' in Parser() optional\n\n* v0.3.0\n\n    * creole2html() has the optional parameter \'blog_line_breaks\' to switch from default blog to wiki line breaks\n\n* v0.2.8\n\n    * bugfix in setup.py\n\n* v0.2.7\n\n    * handle obsolete non-closed <br> tag\n\n* v0.2.6\n\n    * bugfix in setup.py\n\n    * Cleanup DocStrings\n\n    * add unittests\n\n* v0.2.5\n\n    * creole2html: Bugfix if "--", "//" etc. stands alone, see also: `issues 12 <http://code.google.com/p/python-creole/issues/detail?id=12>`_\n\n    * Note: bold, italic etc. can\'t cross line any more.\n\n* v0.2.4\n\n    * creole2html: ignore file extensions in image tag\n\n        * see also: `issues 7 <http://code.google.com/p/python-creole/issues/detail?id=7>`_\n\n* v0.2.3\n\n    * html2creole bugfix/enhanced: convert image tag without alt attribute:\n\n        * see also: `issues 6 <http://code.google.com/p/python-creole/issues/detail?id=6>`_\n\n        * Thanks Betz Stefan alias \'encbladexp\'\n\n* v0.2.2\n\n    * html2creole bugfix: convert ``<a href="/url/">Search & Destroy</a>``\n\n* v0.2.1\n\n    * html2creole bugfixes in:\n\n        * converting tables: ignore tbody tag and better handling p and a tags in td\n\n        * converting named entity\n\n* v0.2\n\n    * remove all django template tag stuff: `issues 3 <http://code.google.com/p/python-creole/issues/detail?id=3>`_\n\n    * html code always escaped\n\n* v0.1.1\n\n    * improve macros stuff, patch by Vitja Makarov: `issues 2 <http://code.google.com/p/python-creole/issues/detail?id=2>`_\n\n* v0.1.0\n\n    * first version cut out from `PyLucid CMS <http://www.pylucid.org>`_\n\n.. |^0.15| image:: ^0.15\n\nfirst source code was written 27.11.2008: `Forum thread (de) <http://www.python-forum.de/viewtopic.php?f=3&t=16742>`_\n\n-------------\nProject links\n-------------\n\n+--------+------------------------------------------------+\n| GitHub | `https://github.com/jedie/python-creole`_      |\n+--------+------------------------------------------------+\n| Wiki   | `https://github.com/jedie/python-creole/wiki`_ |\n+--------+------------------------------------------------+\n| PyPi   | `https://pypi.org/project/python-creole/`_     |\n+--------+------------------------------------------------+\n\n.. _https://github.com/jedie/python-creole: https://github.com/jedie/python-creole\n.. _https://pypi.org/project/python-creole/: https://pypi.org/project/python-creole/\n\n--------\ndonation\n--------\n\n* `paypal.me/JensDiemer <https://www.paypal.me/JensDiemer>`_\n\n* `Flattr This! <https://flattr.com/submit/auto?uid=jedie&url=https%3A%2F%2Fgithub.com%2Fjedie%2Fpython-creole%2F>`_\n\n* Send `Bitcoins <http://www.bitcoin.org/>`_ to `1823RZ5Md1Q2X5aSXRC5LRPcYdveCiVX6F <https://blockexplorer.com/address/1823RZ5Md1Q2X5aSXRC5LRPcYdveCiVX6F>`_\n\n------------\n\n``Note: this file is generated from README.creole 2020-11-04 08:46:39 with "python-creole"``',
    'author': 'Jens Diemer',
    'author_email': 'python-creole@jensdiemer.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jedie/python-creole/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0.0',
}


setup(**setup_kwargs)



"""
    python-creole
    ~~~~~~~~~~~~~


    :copyleft: 2008-2020 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import inspect
import warnings

from creole.shared.utils import dict2string


class DocNode:
    """
    A node in the document tree for html2creole and creole2html.

    The Document tree would be created in the parser and used in the emitter.
    """

    def __init__(self, kind='', parent=None, content=None, attrs=[], level=None):
        self.kind = kind

        self.children = []
        self.parent = parent
        if self.parent is not None:
            self.parent.children.append(self)

        self.attrs = dict(attrs)
        if content:
            assert isinstance(content, str), f"Given content {content!r} is not unicode, it's type: {type(content)}"

        self.content = content
        self.level = level

    def get_attrs_as_string(self):
        """
        FIXME: Find a better was to do this.

        >>> node = DocNode(attrs={'foo':"bar", "no":123})
        >>> node.get_attrs_as_string()
        'foo="bar" no=123'

        >>> node = DocNode(attrs={"foo":'bar', "no":"ABC"})
        >>> node.get_attrs_as_string()
        'foo="bar" no="ABC"'
        """
        return dict2string(self.attrs)

    def __str__(self):
        return str(self.__repr__())

    def __repr__(self):
        return f"<DocNode {self.kind}: {self.content!r}>"
#        return "<DocNode %s (parent: %r): %r>" % (self.kind, self.parent, self.content)

    def debug(self):
        """
        >>> node = DocNode(kind="test", content="foo", attrs={"a":1}, level=0)
        >>> node.debug() # doctest: +NORMALIZE_WHITESPACE +REPORT_NDIFF
        ________________________________________________________________________________
            DocNode - debug:
        str(): <DocNode test: 'foo'>
        attributes:
                       attrs: {'a': 1}
                    children: []
                     content: 'foo'
                        kind: 'test'
                       level: 0
                      parent: None
        """
        print("_" * 80)
        print("\tDocNode - debug:")
        print(f"str(): {self}")
        print("attributes:")
        for i in dir(self):
            if i.startswith("_") or i == "debug":
                continue
            if callable(getattr(self, i)):
                continue
            print(f"{i:>20}: {getattr(self, i, '---')!r}")


class DebugList(list):
    def __init__(self, html2creole):
        self.html2creole = html2creole
        super().__init__()

    def append(self, item):
        #        for stack_frame in inspect.stack(): print(stack_frame)

        line, method = inspect.stack()[1][2:4]
        msg = "%-8s   append: %-35r (%-15s line:%s)" % (
            self.html2creole.getpos(), item,
            method, line
        )
        warnings.warn(msg)
        list.append(self, item)


if __name__ == '__main__':
    import doctest
    print(doctest.testmod())



"""
    creole2html unittest
    ~~~~~~~~~~~~~~~~~~~~

    Here are only some tests witch doesn't work in the cross compare tests.

    Info: There exist some situations with different whitespace handling
        between creol2html and html2creole.

    Test the creole markup.

    :copyleft: 2008-2020 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import sys
import unittest
from io import StringIO

from creole import creole2html
from creole.shared import example_macros
from creole.shared.utils import dict2string, string2dict
from creole.tests import test_macros
from creole.tests.utils.base_unittest import BaseCreoleTest


try:
    import pygments  # noqa flake8
    PYGMENTS = True
except ImportError:
    PYGMENTS = False


class TestCreole2html(BaseCreoleTest):
    """
    Tests around creole2html API and macro function.
    """

    def setUp(self):
        # For fallback tests
        example_macros.PYGMENTS = PYGMENTS

    def test_stderr(self):
        """
        Test if the traceback information send to a stderr handler.
        """
        my_stderr = StringIO()
        creole2html(
            markup_string="<<notexist1>><<notexist2>><</notexist2>>",
            emitter_kwargs={
                "verbose": 2,
                "stderr": my_stderr,
            }
        )
        error_msg = my_stderr.getvalue()

        # Note:
        # The error message change if macros are a dict or are a object!

        # Check if we get a traceback information into our stderr handler
        must_have = (
            "Traceback", "'notexist1'", "'notexist2'",
        )
        for part in must_have:
            tb_lines = [" -" * 40]
            tb_lines += error_msg.splitlines()
            tb_lines += [" -" * 40]
            tb = "\n".join([" >>> %s" % line for line in tb_lines])
            msg = f"{part!r} not found in:\n{tb}"
            # TODO: use assertIn if python 2.6 will be not support anymore.
            if part not in error_msg:
                raise self.failureException(msg)

    def test_example_macros1(self):
        """
        Test the default "html" macro, found in ./creole/default_macros.py
        """
        html = creole2html(
            markup_string="<<html>><p>foo</p><</html>><bar?>",
            emitter_kwargs={
                "verbose": 1,
                "macros": example_macros,
                "stderr": sys.stderr,
            }
        )
        self.assertEqual(html, '<p>foo</p>\n<p>&lt;bar?&gt;</p>')

    def test_example_macros2(self):
        html = creole2html(
            markup_string="<<html>>{{{&lt;nocode&gt;}}}<</html>>",
            emitter_kwargs={
                "verbose": 1,
                "macros": example_macros,
                "stderr": sys.stderr,
            }
        )
        self.assertEqual(html, '{{{&lt;nocode&gt;}}}')

    def test_example_macros3(self):
        html = creole2html(
            markup_string="<<html>>1<</html>><<html>>2<</html>>",
            emitter_kwargs={
                "verbose": 1,
                "macros": example_macros,
                "stderr": sys.stderr,
            }
        )
        self.assertEqual(html, '1\n2')

    def test_macro_dict(self):
        """
        simple test for the "macro API"
        """
        def test(text, foo, bar):
            return "|".join([foo, bar, text])

        html = creole2html(
            markup_string="<<test bar='b' foo='a'>>c<</test>>",
            emitter_kwargs={
                "verbose": 1,
                "macros": {"test": test},
                "stderr": sys.stderr,
            }
        )
        self.assertEqual(html, 'a|b|c')

    def test_macro_callable(self):
        """
        simple test for the "macro API"
        """
        def testmacro():
            pass

        self.assertRaises(TypeError,
                          creole2html,
                          markup_string="<<test no=1 arg2='foo'>>bar<</test>>",
                          emitter_kwargs={
                              "verbose": 1,
                              "macros": testmacro,
                              "stderr": sys.stderr,
                          }
                          )

    def test_macro_wrong_arguments_with_error_report(self):
        """
        simple test for the "macro API"
        """
        def test(text, foo):
            pass
        my_stderr = StringIO()

        html = creole2html(
            markup_string="<<test bar='foo'>>c<</test>>",
            emitter_kwargs={
                "verbose": 2,
                "macros": {"test": test},
                "stderr": my_stderr,
            }
        )
        self.assertEqual(html,
                         "[Error: Macro 'test' error: test() got an unexpected keyword argument 'bar']"
                         )
        error_msg = my_stderr.getvalue()

        # Check traceback information into our stderr handler
        must_have = (
            "TypeError: test() got an unexpected keyword argument 'bar'",
            "sourceline: 'def test(text, foo):' from",
            "tests/test_creole2html.py",
        )
        for part in must_have:
            self.assertIn(part, error_msg)

    def test_macro_wrong_arguments_quite(self):
        """
        simple test for the "macro API"
        """
        def test(text, foo):
            pass
        my_stderr = StringIO()

        html = creole2html(
            markup_string="<<test bar='foo'>>c<</test>>",
            emitter_kwargs={
                "verbose": 1,
                "macros": {"test": test},
                "stderr": my_stderr,
            }
        )
        self.assertEqual(html,
                         "[Error: Macro 'test' error: test() got an unexpected keyword argument 'bar']"
                         )
        error_msg = my_stderr.getvalue()
        self.assertEqual(error_msg, "")

    @unittest.skipIf(not PYGMENTS, "Pygments not installed")
    def test_code_macro(self):
        # due to https://bitbucket.org/birkenfeld/pygments-main/issues/1254/empty-at-the-begining-of-the-highlight
        # an empty <span></span> is now part of pygments output
        self.assert_creole2html(r"""
            Here a simple code macro test:
            <<code ext=".py">>
            for i in xrange(10):
                print('hello world')
            <</code>>
            """, """
            <p>Here a simple code macro test:</p>
            <div class="pygments"><pre><span></span><span class="k">for</span> <span class="n">i</span> <span class="ow">in</span> <span class="n">xrange</span><span class="p">(</span><span class="mi">10</span><span class="p">):</span><br />
                <span class="nb">print</span><span class="p">(</span><span class="s1">&#39;hello world&#39;</span><span class="p">)</span><br />
            </pre></div><br />
            """,
                                macros={'code': example_macros.code}
                                )

    def test_code_macro_fallback(self):
        # force to use fallback. Will be reset in self.setUp()
        example_macros.PYGMENTS = False

        self.assert_creole2html(
            r"""
            Here a simple code macro test:
            <<code ext=".py">>
            for i in xrange(10):
                print('hello world')
            <</code>>
            """, """
            <p>Here a simple code macro test:</p>
            <pre>for i in xrange(10):
                print('hello world')</pre>
            """,
            macros={'code': example_macros.code}
        )

    def test_code_macro_fallback_escape(self):
        # force to use fallback. Will be reset in self.setUp()
        example_macros.PYGMENTS = False

        self.assert_creole2html(
            r"""
            <<code ext=".py">>
            print('This >>should<< be escaped!')
            <</code>>
            """, """
            <pre>print('This &gt;&gt;should&lt;&lt; be escaped!')</pre>
            """,
            macros={'code': example_macros.code}
        )


class TestCreole2htmlMarkup(BaseCreoleTest):

    def test_creole_basic(self):
        out_string = creole2html("a text line.")
        self.assertEqual(out_string, "<p>a text line.</p>")

    def test_lineendings(self):
        """ Test all existing lineending version """
        out_string = creole2html("first\nsecond")
        self.assertEqual(out_string, "<p>first<br />\nsecond</p>")

        out_string = creole2html("first\rsecond")
        self.assertEqual(out_string, "<p>first<br />\nsecond</p>")

        out_string = creole2html("first\r\nsecond")
        self.assertEqual(out_string, "<p>first<br />\nsecond</p>")

    # --------------------------------------------------------------------------

    def test_creole_linebreak(self):
        self.assert_creole2html(r"""
            Force\\linebreak
        """, """
            <p>Force<br />
            linebreak</p>
        """)

    def test_html_lines(self):
        self.assert_creole2html(r"""
            This is a normal Text block witch would
            escape html chars like < and > ;)

            So you can't insert <html> directly.

            <p>This escaped, too.</p>
        """, """
            <p>This is a normal Text block witch would<br />
            escape html chars like &lt; and &gt; ;)</p>

            <p>So you can't insert &lt;html&gt; directly.</p>

            <p>&lt;p&gt;This escaped, too.&lt;/p&gt;</p>
        """)

    def test_escape_char(self):
        self.assert_creole2html(r"""
            ~#1
            http://domain.tld/~bar/
            ~http://domain.tld/
            [[Link]]
            ~[[Link]]
        """, """
            <p>#1<br />
            <a href="http://domain.tld/~bar/">http://domain.tld/~bar/</a><br />
            http://domain.tld/<br />
            <a href="Link">Link</a><br />
            [[Link]]</p>
        """)

    def test_cross_paragraphs(self):
        self.assert_creole2html(r"""
            Bold and italics should //not be...

            ...able// to **cross

            paragraphs.**
        """, """
            <p>Bold and italics should //not be...</p>

            <p>...able// to **cross</p>

            <p>paragraphs.**</p>
        """)

    def test_list_special(self):
        """
        optional whitespace before the list
        """
        self.assert_creole2html(r"""
            * Item 1
            ** Item 1.1
             ** Item 1.2
                ** Item 1.3
                    * Item2

                # one
              ## two
        """, """
        <ul>
            <li>Item 1
            <ul>
                <li>Item 1.1</li>
                <li>Item 1.2</li>
                <li>Item 1.3</li>
            </ul></li>
            <li>Item2</li>
        </ul>
        <ol>
            <li>one
            <ol>
                <li>two</li>
            </ol></li>
        </ol>
        """)

    def test_macro_basic(self):
        """
        Test the three different macro types with a "unittest macro"
        """
        self.assert_creole2html(r"""
            There exist three different macro types:
            A <<unittest_macro1 args="foo1">>bar1<</unittest_macro1>> in a line...
            ...a single <<unittest_macro1 foo="bar">> tag,
            or: <<unittest_macro1 a=1 b=2 />> closed...

            a macro block:
            <<unittest_macro2 char="|">>
            the
            text
            <</unittest_macro2>>
            the end
        """, r"""
            <p>There exist three different macro types:<br />
            A [test macro1 - kwargs: args="foo1",text="bar1"] in a line...<br />
            ...a single [test macro1 - kwargs: foo="bar",text=null] tag,<br />
            or: [test macro1 - kwargs: a=1,b=2,text=null] closed...</p>

            <p>a macro block:</p>
            the|text
            <p>the end</p>
        """,
                                macros=test_macros,
                                )

    def test_macro_html1(self):
        self.assert_creole2html(r"""
                html macro:
                <<html>>
                <p><<this is broken 'html', but it will be pass throu>></p>
                <</html>>

                inline: <<html>>&#x7B;...&#x7D;<</html>> code
            """, r"""
                <p>html macro:</p>
                <p><<this is broken 'html', but it will be pass throu>></p>

                <p>inline: &#x7B;...&#x7D; code</p>
            """,
                                macros=example_macros,
                                )

    def test_macro_not_exist1(self):
        """
        not existing macro with creole2html.HtmlEmitter(verbose=1):
        A error message should be insertet into the generated code

        Two tests: with verbose=1 and verbose=2, witch write a Traceback
        information to a given "stderr"
        """
        source_string = r"""
            macro block:
            <<notexists>>
            foo bar
            <</notexists>>

            inline macro:
            <<notexisttoo foo="bar">>
        """
        should_string = r"""
            <p>macro block:</p>
            [Error: Macro 'notexists' doesn't exist]

            <p>inline macro:<br />
            [Error: Macro 'notexisttoo' doesn't exist]
            </p>
        """

        self.assert_creole2html(source_string, should_string, verbose=1)

        # ----------------------------------------------------------------------
        # Test with verbose=2 ans a StringIO stderr handler

    def test_wrong_macro_syntax(self):
        self.assert_creole2html(r"""
                wrong macro line:
                <<summary>Some funky page summary.<</summary>>
            """, r"""
                <p>wrong macro line:<br />
                [Error: Wrong macro arguments: ">Some funky page summary.<</summary" for macro 'summary' (maybe wrong macro tag syntax?)]
                </p>
            """,  # verbose=True
                                )

    def test_macro_not_exist2(self):
        """
        not existing macro with creole2html.HtmlEmitter(verbose=0):

        No error messages should be inserted.
        """
        self.assert_creole2html(r"""
            macro block:
            <<notexists>>
            foo bar
            <</notexists>>

            inline macro:
            <<notexisttoo foo="bar">>
        """, r"""
            <p>macro block:</p>

            <p>inline macro:<br />
            </p>
        """, verbose=False
                                )

    def test_toc_simple(self):
        """
        Simple test to check the table of content is correctly generated.
        """
        self.assert_creole2html(r"""
            <<toc>>
            = Headline
        """, """
            <ul>
                <li><a href="#Headline">Headline</a></li>
            </ul>
            <a name="Headline"><h1>Headline</h1></a>
        """)

    def test_toc_more_headlines(self):
        self.assert_creole2html(r"""
            Between text and toc must be a newline.

            <<toc>>
            = Headline 1
            == Sub-Headline 1.1
            == Sub-Headline 1.2
            = Headline 2
            == Sub-Headline 2.1
            == Sub-Headline 2.2
        """, """
            <p>Between text and toc must be a newline.</p>

            <ul>
                <li><a href="#Headline 1">Headline 1</a></li>
                <ul>
                    <li><a href="#Sub-Headline 1.1">Sub-Headline 1.1</a></li>
                    <li><a href="#Sub-Headline 1.2">Sub-Headline 1.2</a></li>
                </ul>
                <li><a href="#Headline 2">Headline 2</a></li>
                <ul>
                    <li><a href="#Sub-Headline 2.1">Sub-Headline 2.1</a></li>
                    <li><a href="#Sub-Headline 2.2">Sub-Headline 2.2</a></li>
                </ul>
            </ul>
            <a name="Headline 1"><h1>Headline 1</h1></a>
            <a name="Sub-Headline 1.1"><h2>Sub-Headline 1.1</h2></a>
            <a name="Sub-Headline 1.2"><h2>Sub-Headline 1.2</h2></a>
            <a name="Headline 2"><h1>Headline 2</h1></a>
            <a name="Sub-Headline 2.1"><h2>Sub-Headline 2.1</h2></a>
            <a name="Sub-Headline 2.2"><h2>Sub-Headline 2.2</h2></a>
        """)

    def test_toc_chaotic_headlines(self):
        self.assert_creole2html(r"""
            <<toc>>
            = level 1
            === level 3
            == level 2
            ==== level 4
            = level 1
        """, """
            <ul>
                <li><a href="#level 1">level 1</a></li>
                <ul>
                    <ul>
                        <li><a href="#level 3">level 3</a></li>
                    </ul>
                    <li><a href="#level 2">level 2</a></li>
                    <ul>
                        <ul>
                            <li><a href="#level 4">level 4</a></li>
                        </ul>
                    </ul>
                </ul>
                <li><a href="#level 1">level 1</a></li>
            </ul>
            <a name="level 1"><h1>level 1</h1></a>
            <a name="level 3"><h3>level 3</h3></a>
            <a name="level 2"><h2>level 2</h2></a>
            <a name="level 4"><h4>level 4</h4></a>
            <a name="level 1"><h1>level 1</h1></a>
        """)

    def test_toc_depth_1(self):
        self.assert_creole2html(r"""
            <<toc depth=1>>
            = Headline 1
            == Sub-Headline 1.1
            === Sub-Sub-Headline 1.1.1
            === Sub-Sub-Headline 1.1.2
            == Sub-Headline 1.2
            = Headline 2
            == Sub-Headline 2.1
            == Sub-Headline 2.2
            === Sub-Sub-Headline 2.2.1
        """, """
            <ul>
                <li><a href="#Headline 1">Headline 1</a></li>
                <li><a href="#Headline 2">Headline 2</a></li>
            </ul>
            <a name="Headline 1"><h1>Headline 1</h1></a>
            <a name="Sub-Headline 1.1"><h2>Sub-Headline 1.1</h2></a>
            <a name="Sub-Sub-Headline 1.1.1"><h3>Sub-Sub-Headline 1.1.1</h3></a>
            <a name="Sub-Sub-Headline 1.1.2"><h3>Sub-Sub-Headline 1.1.2</h3></a>
            <a name="Sub-Headline 1.2"><h2>Sub-Headline 1.2</h2></a>
            <a name="Headline 2"><h1>Headline 2</h1></a>
            <a name="Sub-Headline 2.1"><h2>Sub-Headline 2.1</h2></a>
            <a name="Sub-Headline 2.2"><h2>Sub-Headline 2.2</h2></a>
            <a name="Sub-Sub-Headline 2.2.1"><h3>Sub-Sub-Headline 2.2.1</h3></a>
        """)

    def test_toc_depth_2(self):
        self.assert_creole2html(r"""
            <<toc depth=2>>
            = Headline 1
            == Sub-Headline 1.1
            === Sub-Sub-Headline 1.1.1
            === Sub-Sub-Headline 1.1.2
            == Sub-Headline 1.2
            = Headline 2
            == Sub-Headline 2.1
            == Sub-Headline 2.2
            === Sub-Sub-Headline 2.2.1
        """, """
            <ul>
                <li><a href="#Headline 1">Headline 1</a></li>
                <ul>
                    <li><a href="#Sub-Headline 1.1">Sub-Headline 1.1</a></li>
                    <li><a href="#Sub-Headline 1.2">Sub-Headline 1.2</a></li>
                </ul>
                <li><a href="#Headline 2">Headline 2</a></li>
                <ul>
                    <li><a href="#Sub-Headline 2.1">Sub-Headline 2.1</a></li>
                    <li><a href="#Sub-Headline 2.2">Sub-Headline 2.2</a></li>
                </ul>
            </ul>
            <a name="Headline 1"><h1>Headline 1</h1></a>
            <a name="Sub-Headline 1.1"><h2>Sub-Headline 1.1</h2></a>
            <a name="Sub-Sub-Headline 1.1.1"><h3>Sub-Sub-Headline 1.1.1</h3></a>
            <a name="Sub-Sub-Headline 1.1.2"><h3>Sub-Sub-Headline 1.1.2</h3></a>
            <a name="Sub-Headline 1.2"><h2>Sub-Headline 1.2</h2></a>
            <a name="Headline 2"><h1>Headline 2</h1></a>
            <a name="Sub-Headline 2.1"><h2>Sub-Headline 2.1</h2></a>
            <a name="Sub-Headline 2.2"><h2>Sub-Headline 2.2</h2></a>
            <a name="Sub-Sub-Headline 2.2.1"><h3>Sub-Sub-Headline 2.2.1</h3></a>
        """)

    def test_toc_depth_3(self):
        self.assert_creole2html(r"""
            <<toc depth=3>>
            = Headline 1
            == Sub-Headline 1.1
            === Sub-Sub-Headline 1.1.1
            === Sub-Sub-Headline 1.1.2
            == Sub-Headline 1.2
            = Headline 2
            == Sub-Headline 2.1
            == Sub-Headline 2.2
            === Sub-Sub-Headline 2.2.1
        """, """
            <ul>
                <li><a href="#Headline 1">Headline 1</a></li>
                <ul>
                    <li><a href="#Sub-Headline 1.1">Sub-Headline 1.1</a></li>
                    <ul>
                        <li><a href="#Sub-Sub-Headline 1.1.1">Sub-Sub-Headline 1.1.1</a></li>
                        <li><a href="#Sub-Sub-Headline 1.1.2">Sub-Sub-Headline 1.1.2</a></li>
                    </ul>
                    <li><a href="#Sub-Headline 1.2">Sub-Headline 1.2</a></li>
                </ul>
                <li><a href="#Headline 2">Headline 2</a></li>
                <ul>
                    <li><a href="#Sub-Headline 2.1">Sub-Headline 2.1</a></li>
                    <li><a href="#Sub-Headline 2.2">Sub-Headline 2.2</a></li>
                    <ul>
                        <li><a href="#Sub-Sub-Headline 2.2.1">Sub-Sub-Headline 2.2.1</a></li>
                    </ul>
                </ul>
            </ul>
            <a name="Headline 1"><h1>Headline 1</h1></a>
            <a name="Sub-Headline 1.1"><h2>Sub-Headline 1.1</h2></a>
            <a name="Sub-Sub-Headline 1.1.1"><h3>Sub-Sub-Headline 1.1.1</h3></a>
            <a name="Sub-Sub-Headline 1.1.2"><h3>Sub-Sub-Headline 1.1.2</h3></a>
            <a name="Sub-Headline 1.2"><h2>Sub-Headline 1.2</h2></a>
            <a name="Headline 2"><h1>Headline 2</h1></a>
            <a name="Sub-Headline 2.1"><h2>Sub-Headline 2.1</h2></a>
            <a name="Sub-Headline 2.2"><h2>Sub-Headline 2.2</h2></a>
            <a name="Sub-Sub-Headline 2.2.1"><h3>Sub-Sub-Headline 2.2.1</h3></a>
        """)

    def test_toc_with_no_toc(self):
        self.assert_creole2html(r"""
            <<toc>>
            = This is the Headline
            Use {{{<<toc>>}}} to insert a table of contents.
        """, """
            <ul>
                <li><a href="#This is the Headline">This is the Headline</a></li>
            </ul>
            <a name="This is the Headline"><h1>This is the Headline</h1></a>
            <p>Use <tt>&lt;&lt;toc&gt;&gt;</tt> to insert a table of contents.</p>
        """)

    def test_toc_more_then_one_toc(self):
        self.assert_creole2html(r"""
            Not here:
            {{{
            print("<<toc>>")
            }}}

            and onle the first:

            <<toc>>

            <<toc>>
            <<toc>>
            = Headline
            == Sub-Headline
        """, """
            <p>Not here:</p>
            <pre>
            print("&lt;&lt;toc&gt;&gt;")
            </pre>

            <p>and onle the first:</p>

            <ul>
                <li><a href="#Headline">Headline</a></li>
                <ul>
                    <li><a href="#Sub-Headline">Sub-Headline</a></li>
                </ul>
            </ul>

            <p>&lt;&lt;toc&gt;&gt;<br />
            &lt;&lt;toc&gt;&gt;</p>
            <a name="Headline"><h1>Headline</h1></a>
            <a name="Sub-Headline"><h2>Sub-Headline</h2></a>
        """)

    def test_toc_headline_before_toc(self):
        self.assert_creole2html(r"""
            = headline
            == sub headline

            <<toc>>

            ok?
        """, """
            <a name="headline"><h1>headline</h1></a>
            <a name="sub headline"><h2>sub headline</h2></a>

            <ul>
                <li><a href="#headline">headline</a></li>
                <ul>
                    <li><a href="#sub headline">sub headline</a></li>
                </ul>
            </ul>

            <p>ok?</p>
        """)

    def test_image(self):
        """ test image tag with different picture text """
        self.assert_creole2html(r"""
            {{foobar1.jpg}}
            {{/path1/path2/foobar2.jpg}}
            {{/path1/path2/foobar3.jpg|foobar3.jpg}}
        """, """
            <p><img src="foobar1.jpg" title="foobar1.jpg" alt="foobar1.jpg" /><br />
            <img src="/path1/path2/foobar2.jpg" title="/path1/path2/foobar2.jpg" alt="/path1/path2/foobar2.jpg" /><br />
            <img src="/path1/path2/foobar3.jpg" title="foobar3.jpg" alt="foobar3.jpg" /></p>
        """)

    def test_image_with_size(self):
        """ test image tag with size dimention (good and bad) """
        self.assert_creole2html(r"""
            {{/path1/path2/foobar3.jpg|foo|160x90}}
            {{/path1/path2/foobar3.jpg|foo| 160 x 90 }}
            {{/path1/path2/foobar3.jpg|foo|160}}
            {{/path1/path2/foobar3.jpg||160x90}}
            {{/path1/path2/foobar3.jpg|foo|}}
        """, """
            <p><img src="/path1/path2/foobar3.jpg" title="foo" alt="foo" width="160" height="90" /><br />
            <img src="/path1/path2/foobar3.jpg" title="foo" alt="foo" width="160" height="90" /><br />
            <img src="/path1/path2/foobar3.jpg" title="foo|160" alt="foo|160" /><br />
            <img src="/path1/path2/foobar3.jpg" title="/path1/path2/foobar3.jpg" alt="/path1/path2/foobar3.jpg" width="160" height="90" /><br />
            <img src="/path1/path2/foobar3.jpg" title="foo|" alt="foo|" /></p>
        """)

    def test_image_with_size_strict(self):
        """ test image tag with size dimention (good and bad) """
        self.assert_creole2html(r"""
            {{/path1/path2/foobar3.jpg|foo|160x90}}
            {{/path1/path2/foobar3.jpg|foo|160}}
        """, """
            <p><img src="/path1/path2/foobar3.jpg" title="foo|160x90" alt="foo|160x90" /><br />
            <img src="/path1/path2/foobar3.jpg" title="foo|160" alt="foo|160" /></p>
        """, strict=True)

    def test_image_unknown_extension(self):
        self.assert_creole2html(r"""
            # {{/path/to/image.ext|image ext}} one
            # {{/no/extension|no extension}} two
            # {{/image.xyz}} tree
        """, """
            <ol>
                <li><img src="/path/to/image.ext" title="image ext" alt="image ext" /> one</li>
                <li><img src="/no/extension" title="no extension" alt="no extension" /> two</li>
                <li><img src="/image.xyz" title="/image.xyz" alt="/image.xyz" /> tree</li>
            </ol>
        """)

    def test_links(self):
        self.assert_creole2html(r"""
            [[/foobar/Creole_(Markup)]]
            [[http://de.wikipedia.org/wiki/Creole_(Markup)|Creole@wikipedia]]
        """, """
            <p><a href="/foobar/Creole_(Markup)">/foobar/Creole_(Markup)</a><br />
            <a href="http://de.wikipedia.org/wiki/Creole_(Markup)">Creole@wikipedia</a></p>
        """)

    def test_standalone_hyperlink(self):
        self.assert_creole2html(r"""
                a link to the http://www.pylucid.org page.
            """, """
                <p>a link to the <a href="http://www.pylucid.org">http://www.pylucid.org</a> page.</p>
            """
                                )

    def test_wiki_style_line_breaks1(self):
        html = creole2html(
            markup_string=self._prepare_text("""
                wiki style
                linebreaks

                ...and not blog styled.
            """),
            blog_line_breaks=False,
            debug=True, verbose=True
        )
        self.assertEqual(html, self._prepare_text("""
            <p>wiki style linebreaks</p>

            <p>...and not blog styled.</p>
        """))

    def test_wiki_style_line_breaks2(self):
        html = creole2html(
            markup_string=self._prepare_text("""
                **one**
                //two//

                * one
                * two
            """),
            blog_line_breaks=False,
            debug=True, verbose=True
        )
        self.assertEqual(html, self._prepare_text("""
            <p><strong>one</strong> <i>two</i></p>

            <ul>
            \t<li>one</li>
            \t<li>two</li>
            </ul>
        """))

    def test_wiki_style_line_breaks3(self):
        html = creole2html(
            markup_string=self._prepare_text("""
                with blog line breaks, every line break would be convertet into<br />
                with wiki style not.

                This is the first line,\\\\and this is the second.

                new line
                block 1

                new line
                block 2

                end
            """),
            blog_line_breaks=False,
        )
        self.assertEqual(html, self._prepare_text("""
            <p>with blog line breaks, every line break would be convertet into&lt;br /&gt; with wiki style not.</p>

            <p>This is the first line,<br />
            and this is the second.</p>

            <p>new line block 1</p>

            <p>new line block 2</p>

            <p>end</p>
        """))

    def test_headline_spaces(self):
        """
        https://code.google.com/p/python-creole/issues/detail?id=15
        """
        html = creole2html(markup_string="== Headline1 == \n== Headline2== ")
        self.assertEqual(html, self._prepare_text("""
            <h2>Headline1</h2>
            <h2>Headline2</h2>
        """))

    def test_tt(self):
        self.assert_creole2html(r"""
            inline {{{<escaped>}}} and {{{ **not strong** }}}...
            ...and ##**strong** Teletyper## ;)
        """, """
            <p>inline <tt>&lt;escaped&gt;</tt> and <tt> **not strong** </tt>...<br />
            ...and <tt><strong>strong</strong> Teletyper</tt> ;)</p>
        """)

    def test_protocol_in_brackets(self):
        self.assert_creole2html(r"""
            My Server ([[ftp://foo/bar]]) is ok.
        """, """
            <p>My Server (<a href="ftp://foo/bar">ftp://foo/bar</a>) is ok.</p>
        """)
        self.assert_creole2html(r"""
            My Server (ftp://foo/bar) is ok.
        """, """
            <p>My Server (ftp://foo/bar) is ok.</p>
        """)

    def test_protocol_with_brackets(self):
        self.assert_creole2html(r"""
            A http://en.wikipedia.org/wiki/Uri_(Island) link.
        """, """
            <p>A <a href="http://en.wikipedia.org/wiki/Uri_(Island)">http://en.wikipedia.org/wiki/Uri_(Island)</a> link.</p>
        """)

    def test_wrong_protocol(self):
        self.assert_creole2html(r"""
            ~ftp://ok
        """, """
            <p>ftp://ok</p>
        """)
        self.assert_creole2html(r"""
            ftp:
        """, """
            <p>ftp:</p>
        """)
        self.assert_creole2html(r"""
            ftp:/
        """, """
            <p>ftp:/</p>
        """)
        self.assert_creole2html(r"""
            missing space.ftp://ok
        """, """
            <p>missing space.ftp://ok</p>
        """)


class TestStr2Dict(unittest.TestCase):
    def test_basic(self):
        self.assertEqual(
            string2dict('key1="value1" key2="value2"'),
            {'key2': 'value2', 'key1': 'value1'}
        )

    def test_bool(self):
        self.assertEqual(
            string2dict('unicode=True'),
            {'unicode': True}
        )

    def test_mixed1(self):
        self.assertEqual(
            string2dict('A="B" C=1 D=1.1 E=True F=False G=None'),
            {'A': 'B', 'C': 1, 'E': True, 'D': '1.1', 'G': None, 'F': False}
        )

    def test_mixed2(self):
        self.assertEqual(
            string2dict('''key1="'1'" key2='"2"' key3="""'3'""" '''),
            {'key3': 3, 'key2': 2, 'key1': 1}
        )


class TestDict2String(unittest.TestCase):
    def test_basic(self):
        self.assertEqual(
            dict2string({'key': 'value'}),
            'key="value"'
        )

    def test_basic2(self):
        self.assertEqual(
            dict2string({'foo': "bar", "no": 123}),
            'foo="bar" no=123'
        )

    def test_basic3(self):
        self.assertEqual(
            dict2string({"foo": 'bar', "no": "ABC"}),
            'foo="bar" no="ABC"'
        )


if __name__ == '__main__':
    unittest.main(
        verbosity=2
    )



from pathlib import Path

from poetry_publish.publish import poetry_publish
from poetry_publish.utils.subprocess_utils import verbose_check_call

import creole
from creole.setup_utils import assert_rst_readme


def publish():
    """
        Publish python-creole to PyPi
        Call this via:
            $ poetry run publish
    """
    package_root = Path(creole.__file__).parent.parent

    # don't publish if README is not up-to-date:
    assert_rst_readme(package_root=package_root, filename='README.creole')

    # don't publish if code style wrong:
    verbose_check_call('make', 'fix-code-style')

    poetry_publish(
        package_root=package_root,
        version=creole.__version__,
        creole_readme=True  # don't publish if README.rst is not up-to-date
    )



"""
    python-creole
    ~~~~~~~~~~~~~

    created by Jens Diemer

    :copyleft: 2009-2011 by the python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


BLOCK_TAGS = (
    "address", "blockquote", "center", "dir", "div", "dl", "fieldset",
    "form",
    "h1", "h2", "h3", "h4", "h5", "h6",
    "hr", "ins", "isindex", "men", "noframes", "noscript",
    "ul", "ol", "li", "table", "th", "tr", "td",
    "p", "pre",
    "br"
)
IGNORE_TAGS = ("tbody",)



"""
    cross compare creole unittest
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Compare all similarities between:
        * creole2html
        * html2creole

    Note: This only works fine if there is no problematic whitespace handling.
        In this case, we must test in test_creole2html.py or test_html2creole.py

    :copyleft: 2008-2011 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import unittest

from creole.shared import example_macros
from creole.shared.unknown_tags import use_html_macro
from creole.tests.utils.base_unittest import BaseCreoleTest


class CrossCompareCreoleTests(BaseCreoleTest):
    def test_typeface(self):
        self.cross_compare_creole(
            creole_string=r"""
                basics:
                **//bold italics//**
                //**bold italics**//
                //This is **also** good.//

                Creole 1.0 optional:
                This is ##monospace## text.
                This is ^^superscripted^^ text.
                This is ,,subscripted,, text.
                This is __underlined__ text.

                own additions:
                This is --small-- and this ~~strikeout~~ ;)
            """,
            html_string="""
                <p>basics:<br />
                <strong><i>bold italics</i></strong><br />
                <i><strong>bold italics</strong></i><br />
                <i>This is <strong>also</strong> good.</i></p>

                <p>Creole 1.0 optional:<br />
                This is <tt>monospace</tt> text.<br />
                This is <sup>superscripted</sup> text.<br />
                This is <sub>subscripted</sub> text.<br />
                This is <u>underlined</u> text.</p>

                <p>own additions:<br />
                This is <small>small</small> and this <del>strikeout</del> ;)</p>
            """
        )

    def test_cross_lines_html2creole(self):
        """ bold/italics cross lines
        see: http://code.google.com/p/python-creole/issues/detail?id=13
        TODO: The way back creole2html doesn't work, see below
        """
        self.assert_html2creole(r"""
            Bold and italics should //be
            able// to **cross
            lines.**
        """, """
            <p>Bold and italics should <i>be<br />
            able</i> to <strong>cross<br />
            lines.</strong></p>
        """)

    def test_small(self):
        """
        http://code.google.com/p/python-creole/issues/detail?id=12#c0
        """
        self.cross_compare_creole(
            creole_string=r"""
                no -- small
                no // italics
                no ** bold
                no ## monospace
                no ^^ superscripted
                no ,, subscripted
                no __ underline
            """,
            html_string="""
                <p>no -- small<br />
                no // italics<br />
                no ** bold<br />
                no ## monospace<br />
                no ^^ superscripted<br />
                no ,, subscripted<br />
                no __ underline</p>
            """,
            debug=False
        )

    def test_link(self):
        self.cross_compare_creole(
            creole_string=r"""
                this is [[/a internal]] link.
                1 [[internal links|link A]] test.
            """,
            html_string="""
                <p>this is <a href="/a internal">/a internal</a> link.<br />
                1 <a href="internal links">link A</a> test.</p>
            """
        )

    def test_bolditalic_links(self):
        self.cross_compare_creole(r"""
            //[[a internal]]//
            **[[Shortcut2|a page2]]**
            //**[[Shortcut3|a page3]]**//
        """, """
            <p><i><a href="a internal">a internal</a></i><br />
            <strong><a href="Shortcut2">a page2</a></strong><br />
            <i><strong><a href="Shortcut3">a page3</a></strong></i></p>
        """)

    def test_pre_contains_braces(self):
        """
        braces, &gt and %lt in a pre area.
        """
        self.cross_compare_creole(
            creole_string=r"""
                === Closing braces in nowiki:

                {{{
                if (x != NULL) {
                  for (i = 0; i < size; i++) {
                    if (x[i] > 0) {
                      x[i]--;
                  }}}
                }}}
                """,
            html_string="""
                <h3>Closing braces in nowiki:</h3>

                <pre>
                if (x != NULL) {
                  for (i = 0; i &lt; size; i++) {
                    if (x[i] &gt; 0) {
                      x[i]--;
                  }}}
                </pre>
            """)

    def test_pre2(self):
        self.cross_compare_creole(r"""
            111

            {{{
            //This// does **not** get [[formatted]]
            }}}
            222

            one

            {{{
            foo

            bar
            }}}
            two
        """, """
            <p>111</p>

            <pre>
            //This// does **not** get [[formatted]]
            </pre>
            <p>222</p>

            <p>one</p>

            <pre>
            foo

            bar
            </pre>
            <p>two</p>
        """)

    def test_pre(self):
        self.cross_compare_creole(r"""
            start

            {{{
            * no list
            <html escaped>
            }}}
            end
        """, """
            <p>start</p>

            <pre>
            * no list
            &lt;html escaped&gt;
            </pre>
            <p>end</p>
        """)

    def test_tt(self):
        self.cross_compare_creole(r"""
            this is ##**strong** Teletyper## ;)
        """, """
            <p>this is <tt><strong>strong</strong> Teletyper</tt> ;)</p>
        """)

    def test_no_inline_headline(self):
        self.cross_compare_creole(
            creole_string=r"""
                = Headline

                === **not** //parsed//

                No == headline == or?
            """,
            html_string="""
                <h1>Headline</h1>

                <h3>**not** //parsed//</h3>

                <p>No == headline == or?</p>
            """
        )

    def test_horizontal_rule(self):
        self.cross_compare_creole(r"""
            one

            ----

            two
        """, """
            <p>one</p>

            <hr />

            <p>two</p>
        """)

    def test_bullet_list(self):
        self.cross_compare_creole(r"""
            * Item 1
            ** Item 1.1
            ** a **bold** Item 1.2
            * Item 2
            ** Item 2.1
            *** [[a link Item 3.1]]
            *** Force\\linebreak 3.2
            *** item 3.3
            *** item 3.4

            up to five levels

            * 1
            ** 2
            *** 3
            **** 4
            ***** 5
        """, """
            <ul>
                <li>Item 1
                <ul>
                    <li>Item 1.1</li>
                    <li>a <strong>bold</strong> Item 1.2</li>
                </ul></li>
                <li>Item 2
                <ul>
                    <li>Item 2.1
                    <ul>
                        <li><a href="a link Item 3.1">a link Item 3.1</a></li>
                        <li>Force<br />
                        linebreak 3.2</li>
                        <li>item 3.3</li>
                        <li>item 3.4</li>
                    </ul></li>
                </ul></li>
            </ul>
            <p>up to five levels</p>

            <ul>
                <li>1
                <ul>
                    <li>2
                    <ul>
                        <li>3
                        <ul>
                            <li>4
                            <ul>
                                <li>5</li>
                            </ul></li>
                        </ul></li>
                    </ul></li>
                </ul></li>
            </ul>
        """)

    def test_number_list(self):
        self.cross_compare_creole(r"""
            # Item 1
            ## Item 1.1
            ## a **bold** Item 1.2
            # Item 2
            ## Item 2.1
            ### [[a link Item 3.1]]
            ### Force\\linebreak 3.2
            ### item 3.3
            ### item 3.4

            up to five levels

            # 1
            ## 2
            ### 3
            #### 4
            ##### 5
        """, """
            <ol>
                <li>Item 1
                <ol>
                    <li>Item 1.1</li>
                    <li>a <strong>bold</strong> Item 1.2</li>
                </ol></li>
                <li>Item 2
                <ol>
                    <li>Item 2.1
                    <ol>
                        <li><a href="a link Item 3.1">a link Item 3.1</a></li>
                        <li>Force<br />
                        linebreak 3.2</li>
                        <li>item 3.3</li>
                        <li>item 3.4</li>
                    </ol></li>
                </ol></li>
            </ol>
            <p>up to five levels</p>

            <ol>
                <li>1
                <ol>
                    <li>2
                    <ol>
                        <li>3
                        <ol>
                            <li>4
                            <ol>
                                <li>5</li>
                            </ol></li>
                        </ol></li>
                    </ol></li>
                </ol></li>
            </ol>
        """,
                                  #        debug = True
                                  )

    def test_big_table(self):
        self.cross_compare_creole(r"""
            A Table...

            |= Headline  |= a other\\headline    |= the **big end**     |
            | a cell     | a **big** cell        | **//bold italics//** |
            | next\\line | No == headline == or? |                      |
            | link test: | a [[/url/|link]] in   | a cell.              |
            |            |                       | empty cells          |
            ...end
        """, """
            <p>A Table...</p>

            <table>
            <tr>
                <th>Headline</th>
                <th>a other<br />
                    headline</th>
                <th>the <strong>big end</strong></th>
            </tr>
            <tr>
                <td>a cell</td>
                <td>a <strong>big</strong> cell</td>
                <td><strong><i>bold italics</i></strong></td>
            </tr>
            <tr>
                <td>next<br />
                    line</td>
                <td>No == headline == or?</td>
                <td></td>
            </tr>
            <tr>
                <td>link test:</td>
                <td>a <a href="/url/">link</a> in</td>
                <td>a cell.</td>
            </tr>
            <tr>
                <td></td>
                <td></td>
                <td>empty cells</td>
            </tr>
            </table>
            <p>...end</p>
        """,
                                  #            debug = True
                                  )

    def test_html_macro_unknown_nodes(self):
        """
        use the <<html>> macro to mask unknown tags.
        Note:
            All cross compare tests use html2creole.HTML_MACRO_UNKNOWN_NODES
        """
        self.cross_compare_creole("""
            111 <<html>><x><</html>>foo<<html>></x><</html>> 222
            333<<html>><x foo1="bar1"><</html>>foobar<<html>></x><</html>>444

            555<<html>><x /><</html>>666
        """, """
            <p>111 <x>foo</x> 222<br />
            333<x foo1="bar1">foobar</x>444</p>

            <p>555<x />666</p>
        """,
                                  # use macro in creole2html emitter:
                                  macros=example_macros,
                                  # escape unknown tags with <<html>> in html2creole emitter:
                                  unknown_emit=use_html_macro,
                                  )

    def test_entities(self):
        self.cross_compare_creole("""
            less-than sign: <
            greater-than sign: >
        """, """
            <p>less-than sign: &lt;<br />
            greater-than sign: &gt;</p>
        """)

#    def test_macro_html1(self):
#        self.cross_compare_creole(r"""
#            <<a_not_existing_macro>>
#
#            <<code>>
#            some code
#            <</code>>
#
#            a macro:
#            <<code>>
#            <<code>>
#            the sourcecode
#            <</code>>
#        """, r"""
#            <p>[Error: Macro 'a_not_existing_macro' doesn't exist]</p>
#            <fieldset class="pygments_code">
#            <legend class="pygments_code"><small title="no lexer matching the text found">unknown type</small></legend>
#            <pre><code>some code</code></pre>
#            </fieldset>
#            <p>a macro:</p>
#            <fieldset class="pygments_code">
#            <legend class="pygments_code"><small title="no lexer matching the text found">unknown type</small></legend>
#            <pre><code>&lt;&lt;code&gt;&gt;
#            the sourcecode</code></pre>
#            </fieldset>
#        """)


#    def test_macro_pygments_code(self):
#        self.cross_compare_creole(r"""
#            a macro:
#            <<code ext=.css>>
#            /* Stylesheet */
#            form * {
#              vertical-align:middle;
#            }
#            <</code>>
#            the end
#        """, """
#            <p>a macro:</p>
#            <fieldset class="pygments_code">
#            <legend class="pygments_code">CSS</legend><table class="pygmentstable"><tr><td class="linenos"><pre>1
#            2
#            3
#            4</pre></td><td class="code"><div class="pygments"><pre><span class="c">/* Stylesheet */</span>
#            <span class="nt">form</span> <span class="o">*</span> <span class="p">{</span>
#              <span class="k">vertical-align</span><span class="o">:</span><span class="k">middle</span><span class="p">;</span>
#            <span class="p">}</span>
#            </pre></div>
#            </td></tr></table></fieldset>
#            <p>the end</p>
#        """)


if __name__ == '__main__':
    unittest.main()



"""
    python-creole commandline interface
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2013-2020 by the python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import argparse
import codecs

from creole import VERSION_STRING, creole2html, html2creole, html2rest, html2textile


class CreoleCLI:
    def __init__(self, convert_func):
        self.convert_func = convert_func
        self.parser = argparse.ArgumentParser(
            description=(
                "python-creole is an open-source (GPL) markup converter"
                " in pure Python for:"
                " creole2html, html2creole, html2ReSt, html2textile"
            ),
        )
        self.parser.add_argument(
            '--version', action='version',
            version='%%(prog)s from python-creole v%s' % VERSION_STRING  # noqa flynt
        )
        self.parser.add_argument("sourcefile", help="source file to convert")
        self.parser.add_argument("destination", help="Output filename")
        self.parser.add_argument("--encoding",
                                 default="utf-8",
                                 help="Codec for read/write file (default encoding: utf-8)"
                                 )

        args = self.parser.parse_args()

        sourcefile = args.sourcefile
        destination = args.destination
        encoding = args.encoding

        self.convert(sourcefile, destination, encoding)

    def convert(self, sourcefile, destination, encoding):
        print(f"Convert {sourcefile!r} to {destination!r} with {self.convert_func.__name__} (codec: {encoding})")

        with codecs.open(sourcefile, "r", encoding=encoding) as infile:
            with codecs.open(destination, "w", encoding=encoding) as outfile:
                content = infile.read()
                converted = self.convert_func(content)
                outfile.write(converted)
        print(f"done. {destination!r} created.")


def cli_creole2html():
    CreoleCLI(creole2html)


def cli_html2creole():
    CreoleCLI(html2creole)


def cli_html2rest():
    CreoleCLI(html2rest)


def cli_html2textile():
    CreoleCLI(html2textile)


if __name__ == "__main__":
    import sys
    sys.argv += ["../README.creole", "../test.html"]
    print(sys.argv)
    cli_creole2html()



"""
    rest2html unittest
    ~~~~~~~~~~~~~~~~~~

    Unittests for rest2html, see: creole/rest2html/clean_writer.py

    :copyleft: 2011-2012 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import tempfile
import unittest

from creole.tests.utils.base_unittest import BaseCreoleTest


class ReSt2HtmlTests(BaseCreoleTest):
    def test_clean_link_table(self):
        self.assert_rest2html("""
            :homepage:
              http://code.google.com/p/python-creole/

            :sourcecode:
              http://github.com/jedie/python-creole
        """, """
            <table>
            <tr><th>homepage:</th><td><a href="http://code.google.com/p/python-creole/">http://code.google.com/p/python-creole/</a></td>
            </tr>
            <tr><th>sourcecode:</th><td><a href="http://github.com/jedie/python-creole">http://github.com/jedie/python-creole</a></td>
            </tr>
            </table>
        """)

    def test_clean_table(self):
        self.assert_rest2html("""
            +------------+------------+
            | Headline 1 | Headline 2 |
            +============+============+
            | cell one   | cell two   |
            +------------+------------+
        """, """
            <table>
            <colgroup>
            <col width="50%" />
            <col width="50%" />
            </colgroup>
            <tr><th>Headline 1</th>
            <th>Headline 2</th>
            </tr>
            <tr><td>cell one</td>
            <td>cell two</td>
            </tr>
            </table>
        """)

    def test_clean_list(self):
        self.assert_rest2html("""
            * item 1

                * item 1.1

                * item 1.2

            * item 2

            numbered list:

            #. item A

            #. item B
        """, """
            <ul>
            <li><p>item 1</p>
            <ul>
            <li>item 1.1</li>
            <li>item 1.2</li>
            </ul>
            </li>
            <li><p>item 2</p>
            </li>
            </ul>
            <p>numbered list:</p>
            <ol>
            <li>item A</li>
            <li>item B</li>
            </ol>
        """)

    def test_clean_headline(self):
        self.assert_rest2html("""
            ======
            head 1
            ======

            ------
            head 2
            ------
        """, """
            <h1>head 1</h1>
            <h2>head 2</h2>
        """)

    def test_include_disabled_by_default(self):
        self.assert_rest2html("""
            Include should be disabled by default.

            .. include:: doesntexist.txt
        """, """
            <p>Include should be disabled by default.</p>
        """, report_level=3)  # Set log level to "error" to suppress the waring output

    def test_include_enabled(self):
        test_content = "Content from include file."
        test_content = test_content.encode("utf-8")
        with tempfile.NamedTemporaryFile() as temp:
            temp.write(test_content)
            temp.flush()
            self.assert_rest2html(f"""
                Enable include and test it.

                .. include:: {temp.name}
            """, """
                <p>Enable include and test it.</p>
                <p>Content from include file.</p>
            """, file_insertion_enabled=True, input_encoding="utf-8")

    def test_raw_disabled_by_default(self):
        self.assert_rest2html("""
            Raw directive should be disabled by default.

            .. raw:: html

               <hr width=50 size=10>
        """, """
            <p>Raw directive should be disabled by default.</p>
        """, report_level=3)  # Set log level to "error" to suppress the waring output

    def test_raw_enabled(self):
        self.assert_rest2html("""
            Now RAW is enabled.

            .. raw:: html

               <hr width=50 size=10>
        """, """
            <p>Now RAW is enabled.</p>
            <hr width=50 size=10>
        """, raw_enabled=True)

    def test_preserve_image_alignment(self):
        self.assert_rest2html("""
            Image alignment should be preserved.

            .. image:: foo.png
               :align: right
        """, """
            <p>Image alignment should be preserved.</p>
            <img alt="foo.png" src="foo.png" align="right" />
        """)

    def test_preserve_figure_alignment(self):
        self.assert_rest2html("""
            Image alignment should be preserved.

            .. figure:: bar.png
               :align: right
        """, """
            <p>Image alignment should be preserved.</p>
            <img alt="bar.png" src="bar.png" align="right" />
        """)


if __name__ == '__main__':
    unittest.main()



"""
    Try to be so strict as PyPi.

    Code based on:
    https://bitbucket.org/pypa/pypi/src/tip/description_utils.py

    see also:
    https://bitbucket.org/pypa/pypi/issue/161/rest-formatting-fails-and-there-is-no-way
"""


from urllib.parse import urlparse

from creole.exceptions import DocutilsImportError


try:
    import docutils  # noqa flake8
    from docutils import io, readers
    from docutils.core import Publisher, publish_doctree
    from docutils.transforms import TransformError
except ImportError as err:
    msg = (
        "%s - You can't use rest2html!"
        " Please install: http://pypi.python.org/pypi/docutils"
    ) % err
    raise DocutilsImportError(msg)


ALLOWED_SCHEMES = '''file ftp gopher hdl http https imap mailto mms news nntp
prospero rsync rtsp rtspu sftp shttp sip sips snews svn svn+ssh telnet
wais irc'''.split()


def pypi_rest2html(source, output_encoding='unicode'):
    """
    >>> pypi_rest2html("test!")
    '<p>test!</p>\\n'
    """
    settings_overrides = {
        'raw_enabled': 0,  # no raw HTML code
        'file_insertion_enabled': 0,  # no file/URL access
        'halt_level': 2,  # at warnings or errors, raise an exception
        'report_level': 5,  # never report problems with the reST code
    }

    # Convert reStructuredText to HTML using Docutils.
    document = publish_doctree(source=source,
                               settings_overrides=settings_overrides)

    for node in document.traverse():
        if node.tagname == '#text':
            continue
        if node.hasattr('refuri'):
            uri = node['refuri']
        elif node.hasattr('uri'):
            uri = node['uri']
        else:
            continue
        o = urlparse(uri)
        if o.scheme not in ALLOWED_SCHEMES:
            raise TransformError('link scheme not allowed')

    # now turn the transformed document into HTML
    reader = readers.doctree.Reader(parser_name='null')
    pub = Publisher(reader, source=io.DocTreeInput(document),
                    destination_class=io.StringOutput)
    pub.set_writer('html')
    pub.process_programmatic_settings(None, settings_overrides, None)
    pub.set_destination(None, None)
    pub.publish()
    parts = pub.writer.parts

    output = parts['body']

    if output_encoding != 'unicode':
        output = output.encode(output_encoding)

    return output


if __name__ == '__main__':
    import doctest
    print(doctest.testmod())



"""
    unittest for CLI
    ~~~~~~~~~~~~~~~~

    :copyleft: 2013-2020 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import sys
import tempfile
import unittest

from creole import VERSION_STRING, cmdline
from creole.tests.utils.base_unittest import BaseCreoleTest
from creole.tests.utils.unittest_subprocess import SubprocessMixin


CMDS = ("creole2html", "html2creole", "html2rest", "html2textile")


class CliTestMixins:
    def test_creole2html(self):
        self._test_convert(
            source_content=b"= test creole2html =",
            dest_content="<h1>test creole2html</h1>",
            cli_str="creole2html",
        )

    def test_html2creole(self):
        self._test_convert(
            source_content=b"<h1>test html2creole</h1>",
            dest_content="= test html2creole",
            cli_str="html2creole",
        )

    def test_html2rest(self):
        self._test_convert(
            source_content=b"<h1>test html2rest</h1>",
            dest_content=(
                "==============\n"
                "test html2rest\n"
                "=============="
            ),
            cli_str="html2rest",
        )

    def test_html2textile(self):
        self._test_convert(
            source_content=b"<h1>test html2textile</h1>",
            dest_content="h1. test html2textile",
            cli_str="html2textile",
        )


class CreoleCLITests(BaseCreoleTest, SubprocessMixin, CliTestMixins):
    def _test_convert(self, source_content, dest_content, cli_str, verbose=True):
        assert isinstance(source_content, bytes), type(source_content)

        source_file = tempfile.NamedTemporaryFile()
        sourcefilepath = source_file.name
        source_file.write(source_content)
        source_file.seek(0)

        dest_file = tempfile.NamedTemporaryFile()
        destfilepath = dest_file.name

        stdout = (
            "Convert '%(src)s' to '%(dst)s' with %(prog)s (codec: utf-8)\n"
            "done. '%(dst)s' created."
        ) % {
            "prog": cli_str,
            "src": sourcefilepath,
            "dst": destfilepath,
        }

        self.assertSubprocess(
            popen_args=[cli_str, sourcefilepath, destfilepath],
            retcode=0, stdout=stdout,
        )

        dest_file.seek(0)
        result_content = dest_file.read()

        result_content = result_content.decode("utf-8")
        self.assertEqual(result_content, dest_content)

    def test_version(self):
        for cmd in CMDS:
            version_info = f"{cmd} from python-creole v{VERSION_STRING}"
            self.assertSubprocess(
                popen_args=[cmd, "--version"],
                retcode=0,
                stdout=version_info,
            )


class CreoleCLITestsDirect(BaseCreoleTest, CliTestMixins):

    def setUp(self):
        super().setUp()
        self._old_sys_argv = sys.argv[:]

    def tearDown(self):
        sys.argv = self._old_sys_argv

    def _test_convert(self, source_content, dest_content, cli_str, verbose=True):
        assert isinstance(source_content, bytes), type(source_content)

        source_file = tempfile.NamedTemporaryFile()
        sourcefilepath = source_file.name
        source_file.write(source_content)
        source_file.seek(0)

        dest_file = tempfile.NamedTemporaryFile()
        destfilepath = dest_file.name

        sys.argv = [cli_str, sourcefilepath, destfilepath]
        cli = getattr(cmdline, f"cli_{cli_str}")
        cli()

        dest_file.seek(0)
        result_content = dest_file.read()

        result_content = result_content.decode("utf-8")
        self.assertEqual(result_content, dest_content)


if __name__ == '__main__':
    unittest.main()



"""
    python-creole
    ~~~~~~~~~~~~~

    :sourcecode:
      https://github.com/jedie/python-creole

    :PyPi:
      https://pypi.org/project/python-creole/

    :copyleft: 2008-2020 by python-creole team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import warnings

from creole.emitter.creol2html_emitter import HtmlEmitter
from creole.emitter.html2creole_emitter import CreoleEmitter
from creole.emitter.html2rest_emitter import ReStructuredTextEmitter
from creole.emitter.html2textile_emitter import TextileEmitter
from creole.parser.creol2html_parser import CreoleParser
from creole.parser.html_parser import HtmlParser


__version__ = "1.4.92"
__api__ = "1.0"  # Creole 1.0 spec - http://wikicreole.org/

VERSION_STRING = __version__  # remove in future
API_STRING = __api__  # remove in future


def creole2html(markup_string, debug=False,
                parser_kwargs=None, emitter_kwargs=None,
                block_rules=None, blog_line_breaks=True,
                macros=None, verbose=None, stderr=None,
                strict=False,
                ):
    """
    convert creole markup into html code

    >>> creole2html('This is **creole //markup//**!')
    '<p>This is <strong>creole <i>markup</i></strong>!</p>'

    Info: parser_kwargs and emitter_kwargs are deprecated
    """
    assert isinstance(markup_string, str), "given markup_string must be unicode!"

    parser_kwargs2 = {
        "block_rules": block_rules,
        "blog_line_breaks": blog_line_breaks,
        "debug": debug,
    }
    if parser_kwargs is not None:
        warnings.warn(
            "parser_kwargs argument in creole2html would be removed in the future!",
            PendingDeprecationWarning)
        parser_kwargs2.update(parser_kwargs)

    # Create document tree from creole markup
    document = CreoleParser(markup_string, **parser_kwargs2).parse()
    if debug:
        document.debug()

    emitter_kwargs2 = {
        "macros": macros,
        "verbose": verbose,
        "stderr": stderr,
        "strict": strict,
    }
    if emitter_kwargs is not None:
        warnings.warn(
            "emitter_kwargs argument in creole2html would be removed in the future!",
            PendingDeprecationWarning)
        emitter_kwargs2.update(emitter_kwargs)

    # Build html code from document tree
    return HtmlEmitter(document, **emitter_kwargs2).emit()


def parse_html(html_string, debug=False):
    """ create the document tree from html code """
    assert isinstance(html_string, str), "given html_string must be unicode!"

    h2c = HtmlParser(debug=debug)
    document_tree = h2c.feed(html_string)
    if debug:
        h2c.debug()
    return document_tree


def html2creole(
    html_string,
    debug=False,
    parser_kwargs=None,
    emitter_kwargs=None,
    unknown_emit=None,
    strict=False,
):
    """
    convert html code into creole markup

    >>> html2creole('<p>This is <strong>creole <i>markup</i></strong>!</p>')
    'This is **creole //markup//**!'
    """
    if parser_kwargs is not None:
        warnings.warn(
            "parser_kwargs argument in html2creole would be removed in the future!",
            PendingDeprecationWarning)

    document_tree = parse_html(html_string, debug=debug)

    emitter_kwargs2 = {
        "unknown_emit": unknown_emit,
        "strict": strict,
    }
    if emitter_kwargs is not None:
        warnings.warn(
            "emitter_kwargs argument in html2creole would be removed in the future!",
            PendingDeprecationWarning)
        emitter_kwargs2.update(emitter_kwargs)

    # create creole markup from the document tree
    emitter = CreoleEmitter(document_tree, debug=debug, **emitter_kwargs2)
    return emitter.emit()


def html2textile(html_string, debug=False,
                 parser_kwargs=None, emitter_kwargs=None,
                 unknown_emit=None
                 ):
    """
    convert html code into textile markup

    >>> html2textile('<p>This is <strong>textile <i>markup</i></strong>!</p>')
    'This is *textile __markup__*!'
    """
    if parser_kwargs is not None:
        warnings.warn(
            "parser_kwargs argument in html2textile would be removed in the future!",
            PendingDeprecationWarning)

    document_tree = parse_html(html_string, debug=debug)

    emitter_kwargs2 = {
        "unknown_emit": unknown_emit,
    }
    if emitter_kwargs is not None:
        warnings.warn(
            "emitter_kwargs argument in html2textile would be removed in the future!",
            PendingDeprecationWarning)
        emitter_kwargs2.update(emitter_kwargs)

    # create textile markup from the document tree
    emitter = TextileEmitter(document_tree, debug=debug, **emitter_kwargs2)
    return emitter.emit()


def html2rest(html_string, debug=False,
              parser_kwargs=None, emitter_kwargs=None,
              unknown_emit=None
              ):
    """
    convert html code into ReStructuredText markup

    >>> html2rest('<p>This is <strong>ReStructuredText</strong> <em>markup</em>!</p>')
    'This is **ReStructuredText** *markup*!'
    """
    if parser_kwargs is not None:
        warnings.warn("parser_kwargs argument in html2rest would be removed in the future!", PendingDeprecationWarning)

    document_tree = parse_html(html_string, debug=debug)

    emitter_kwargs2 = {
        "unknown_emit": unknown_emit,
    }
    if emitter_kwargs is not None:
        warnings.warn(
            "emitter_kwargs argument in html2rest would be removed in the future!",
            PendingDeprecationWarning)
        emitter_kwargs2.update(emitter_kwargs)

    # create ReStructuredText markup from the document tree
    emitter = ReStructuredTextEmitter(document_tree, debug=debug, **emitter_kwargs2)
    return emitter.emit()
